@@ -1704,7 +1704,15 @@ void ls::user_space_consumer_channel_keys::iterator::_skip_to_next_app_per_pid(
 			return;
 		}
 
-		const auto& app = *lttng::utils::container_of(app_node, &ust_app::pid_n);
+		auto& app = *lttng::utils::container_of(app_node, &ust_app::pid_n);
+		if (!ust_app_get(app)) {
+			/* Application is being torn down. Skip it. */
+			continue;
+		}
+
+		/* Hold a reference on the current app to prevent its teardown. */
+		_position._per_pid.current_app = ust_app_reference{ &app };
+
 		auto app_session = ust_app_lookup_app_session(&_creation_context._session, &app);
 
 		if (!app_session) {
@@ -1789,10 +1797,10 @@ void ls::user_space_consumer_channel_keys::iterator::_advance_one_per_uid()
 
 		/* Assumes a registry can't be empty. */
 		LTTNG_ASSERT(cds_lfht_iter_get_node(&_position.channel_iterator.iter));
+	} else {
+		cds_lfht_next(position.current_registry->registry->channels->ht,
+			      &_position.channel_iterator.iter);
 	}
-
-	cds_lfht_next(position.current_registry->registry->channels->ht,
-		      &_position.channel_iterator.iter);
 }
 
 bool ls::user_space_consumer_channel_keys::iterator::operator==(const iterator& other) const noexcept
