@@ -18,11 +18,12 @@
 #include <common/compat/string.hpp>
 #include <common/dynamic-buffer.hpp>
 #include <common/exception.hpp>
-#include <common/file-descriptor.hpp>
 #include <common/format.hpp>
+#include <common/make-unique-wrapper.hpp>
 #include <common/readwrite.hpp>
 #include <common/runas.hpp>
 #include <common/scope-exit.hpp>
+#include <common/stream-descriptor.hpp>
 #include <common/string-utils/c-string-view.hpp>
 #include <common/string-utils/format.hpp>
 
@@ -30,6 +31,7 @@
 
 #include <algorithm>
 #include <ctype.h>
+#include <dirent.h>
 #include <fcntl.h>
 #include <grp.h>
 #include <inttypes.h>
@@ -40,6 +42,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <vector>
 
 #define PROC_MEMINFO_PATH	       "/proc/meminfo"
 #define PROC_MEMINFO_MEMAVAILABLE_LINE "MemAvailable:"
@@ -1396,12 +1399,8 @@ end:
 namespace {
 unsigned int get_max_possible_cpu_id() LTTNG_MAY_THROW
 {
-	constexpr unsigned int cpu_mask_buffer_length{ DEFAULT_LINUX_POSSIBLE_CPU_MASK_LENGTH };
-	std::array<uint8_t, cpu_mask_buffer_length> possible_cpu_mask{};
-	static int max_possible_cpu_id{ 0 };
-	size_t possible_cpu_mask_len{ 0 };
-	unsigned long cpu_index{ 0 };
-	int i{};
+	constexpr auto cpu_mask_buffer_length{ DEFAULT_LINUX_POSSIBLE_CPU_MASK_LENGTH };
+	static unsigned int max_possible_cpu_id = 0;
 
 	if (max_possible_cpu_id != 0) {
 		return max_possible_cpu_id;
@@ -1411,47 +1410,60 @@ unsigned int get_max_possible_cpu_id() LTTNG_MAY_THROW
 		const auto raw_handle = open(DEFAULT_LINUX_POSSIBLE_CPU_PATH, O_RDONLY);
 		if (raw_handle < 0) {
 			LTTNG_THROW_POSIX(
-				lttng::format("Failed to open possible CPU file, path='{}'",
+				lttng::format("Failed to open possible CPU file, path=`{}`",
 					      DEFAULT_LINUX_POSSIBLE_CPU_PATH),
 				errno);
 		}
-		return lttng::file_descriptor(raw_handle);
+
+		return lttng::input_stream_descriptor(raw_handle);
 	}();
 
-	const ssize_t bytes_read =
-		read(possible_cpu_mask_fd.fd(), possible_cpu_mask.data(), cpu_mask_buffer_length);
+	std::array<char, cpu_mask_buffer_length> possible_cpu_mask;
+	possible_cpu_mask.fill(0);
+
+	const auto bytes_read =
+		possible_cpu_mask_fd.read_some(possible_cpu_mask.data(), cpu_mask_buffer_length);
 	if (bytes_read == cpu_mask_buffer_length) {
-		uint8_t next{};
-		if (read(possible_cpu_mask_fd.fd(), &next, 1) != 0) {
-			LTTNG_THROW_ERROR(lttng::format(
-				"Possible CPU mask length exceeds maximum configured size: path='{}', max_size={}",
-				DEFAULT_LINUX_POSSIBLE_CPU_PATH,
-				cpu_mask_buffer_length));
-		}
+		LTTNG_THROW_ERROR(lttng::format(
+			"Possible CPU mask length exceeds or meets maximum configured size: path=`{}`, max_size={}",
+			DEFAULT_LINUX_POSSIBLE_CPU_PATH,
+			cpu_mask_buffer_length));
 	}
 
-	possible_cpu_mask_len = (size_t) bytes_read;
+	auto possible_cpu_mask_len = bytes_read;
 	if (possible_cpu_mask_len < 1) {
-		LTTNG_THROW_ERROR(lttng::format("0 bytes read from possible cpu file path={}",
+		LTTNG_THROW_ERROR(lttng::format("0 bytes read from possible cpu file path=`{}`",
 						DEFAULT_LINUX_POSSIBLE_CPU_PATH));
 	}
 
 	/* Start from the end to read the last CPU index. */
-	i = possible_cpu_mask_len;
-	for (auto iter = possible_cpu_mask.crbegin(); iter != possible_cpu_mask.crend(); iter++) {
+	int i = possible_cpu_mask_len;
+	while (i-- > 0) {
 		/* Break when we hit the first separator. */
 		if ((possible_cpu_mask[i] == ',') || (possible_cpu_mask[i] == '-')) {
 			i++;
 			break;
 		}
-		i--;
 	}
 
-	cpu_index = strtoul((const char *) &possible_cpu_mask.data()[i],
-			    (char **) &possible_cpu_mask.data()[possible_cpu_mask_len],
-			    10);
-	if ((i != possible_cpu_mask_len) && (cpu_index < INT_MAX)) {
-		max_possible_cpu_id = (unsigned int) cpu_index;
+	DBG_FMT("Read possible CPU mask: mask_contents=`{}`",
+		lttng::c_string_view(possible_cpu_mask.data()));
+
+	const char *cpu_id_parse_start = &possible_cpu_mask.data()[i];
+	char *end_ptr;
+	unsigned long cpu_id =
+		strtoul(cpu_id_parse_start,
+			&end_ptr,
+			10);
+	if (cpu_id == 0 && end_ptr == cpu_id_parse_start) {
+		/* No digits found. */
+		LTTNG_THROW_ERROR(lttng::format(
+			"Failed to parse possible CPU mask: mask_contents=`{}`",
+			lttng::c_string_view(possible_cpu_mask.data())));
+	}
+
+	if ((i != possible_cpu_mask_len) && (cpu_id < UINT_MAX)) {
+		max_possible_cpu_id = static_cast<unsigned int>(cpu_id);
 		return max_possible_cpu_id;
 	}
 
@@ -1530,3 +1542,78 @@ bool utils_force_experimental_ctf_2()
 
 	return false;
 }
+
+static void lttng_closedir(DIR *d)
+{
+	if (closedir(d)) {
+		PWARN_FMT("Failed to close directory");
+	}
+}
+
+std::vector<int> list_open_fds()
+{
+	std::vector<int> results;
+	const char *proc_self_fd = "/proc/self/fd";
+	const auto self_fd_handle =
+		lttng::make_unique_wrapper<DIR, lttng_closedir>(opendir(proc_self_fd));
+
+	if (!self_fd_handle) {
+		LTTNG_THROW_POSIX(fmt::format("Failed to open directory: path=`{}`", proc_self_fd),
+				  errno);
+	}
+
+	const auto dir_fd = dirfd(self_fd_handle.get());
+	if (dir_fd < 0) {
+		LTTNG_THROW_POSIX(fmt::format("Failed to get directory file descriptor: path=`{}`",
+					      proc_self_fd),
+				  errno);
+	}
+
+	struct dirent *entry = nullptr;
+	while ((entry = readdir(self_fd_handle.get())) != nullptr) {
+		errno = 0;
+		char *end_ptr = nullptr;
+		const auto fd = static_cast<int>(strtol(entry->d_name, &end_ptr, 10));
+
+		/*
+		 * Skip '.' and '..'.
+		 *
+		 * Not all file systems support setting entry->d_type, and to filter out
+		 * directories systematically an additional lstat() call would be required
+		 * for each entry.
+		 */
+		if (entry->d_name[0] == '.') {
+			continue;
+		}
+
+		/*
+		 * Since strtol() _may_ set errno to EINVAL when no conversion is performed,
+		 * the both errno and the value of end_ptr are checked to determine
+		 * if conversion was successful.
+		 */
+		if (errno || (end_ptr != nullptr && *end_ptr != '\0')) {
+			errno = errno ? errno : EINVAL;
+			PWARN_FMT(
+				"Failed to convert file name to file descriptor when enumerating open files: file_name=`{}`, end_ptr=`{}`",
+				entry->d_name,
+				end_ptr);
+			continue;
+		}
+
+		if (fd == dir_fd) {
+			/* Skip the directory file descriptor we are iterating on. */
+			continue;
+		}
+
+		if (fd < 0) {
+			DBG_FMT("Skipping entry in `{}` that converted to a negative integer value: file_name=`{}`",
+				proc_self_fd,
+				entry->d_name);
+			continue;
+		}
+
+		results.push_back(fd);
+	}
+
+	return results;
+}
