@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+ * SPDX-FileCopyrightText: 2011 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
  * SPDX-License-Identifier: MIT
  *
@@ -13,13 +13,10 @@
 
 #include <lttng/tracepoint.h>
 
-TRACEPOINT_EVENT(tp, slow,
-	TP_ARGS(unsigned int, c, const char *, thread_name),
-	TP_FIELDS(
-		ctf_integer(unsigned int, counter, c)
-		ctf_string(th_name, thread_name)
-	)
-)
+TRACEPOINT_EVENT(tp,
+		 slow,
+		 TP_ARGS(unsigned int, c, const char *, thread_name),
+		 TP_FIELDS(ctf_integer(unsigned int, counter, c) ctf_string(th_name, thread_name)))
 
 #endif /* _TRACEPOINT_TP_H */
 
