@@ -1,3 +1,10 @@
+/*
+ * SPDX-FileCopyrightText: 2016 Jérémie Galarneau <jeremie.galarneau@efficios.com>
+ *
+ * SPDX-License-Identifier: MIT
+ *
+ */
+
 #undef TRACEPOINT_PROVIDER
 #define TRACEPOINT_PROVIDER libzzz
 
@@ -9,12 +16,7 @@
 
 #include <lttng/tracepoint.h>
 
-TRACEPOINT_EVENT(
-    libzzz,
-    zzz,
-    TP_ARGS(void),
-    TP_FIELDS()
-)
+TRACEPOINT_EVENT(libzzz, zzz, TP_ARGS(void), TP_FIELDS())
 
 #endif /* _LIBZZZ_TP_H */
 
