@@ -5,6 +5,7 @@
 # SPDX-License-Identifier: GPL-2.0-only
 #
 
+import multiprocessing
 from types import FrameType
 from typing import Callable, Iterator, Optional, Tuple, List, Generator
 import sys
@@ -1249,6 +1250,107 @@ def getconf(name):
     return p.stdout.read().decode("utf-8").strip()
 
 
+def count_events(trace_path, ignore_exceptions=False):
+    """
+    Returns a tuple of (received events, discarded events) from the trace path.
+
+    trace_path may be an iterable (e.g., array, map from pathlib.Path.glob()).
+    """
+    received = 0
+    discarded = 0
+    dirs = []
+    if isinstance(trace_path, str) or isinstance(trace_path, pathlib.Path):
+        dirs.append(str(trace_path))
+    else:
+        dirs = trace_path
+
+    for dir in dirs:
+        try:
+            for msg in bt2.TraceCollectionMessageIterator(str(dir)):
+                if type(msg) is bt2._EventMessageConst:
+                    received += 1
+                if type(msg) is bt2._DiscardedEventsMessageConst:
+                    # msg.count may be None when the value is indeterminate
+                    discarded += msg.count if msg.count is not None else 1
+        except RuntimeError as e:
+            if not ignore_exceptions:
+                raise e
+
+    return (received, discarded)
+
+
+def count_events_worker(args):
+    trace_path = args
+    received = 0
+    discarded = 0
+    for msg in bt2.TraceCollectionMessageIterator(str(trace_path)):
+        if type(msg) is bt2._EventMessageConst:
+            received += 1
+            continue
+
+        if type(msg) is bt2._DiscardedEventsMessageConst:
+            discarded += msg.count if msg.count is not None else 1
+
+    return (received, discarded)
+
+
+def parallel_count_events(trace_path, test_env):
+    """
+    Count the number of events in a trace using parallel processing.
+    This is a workaround for the performance issues with bt2 bindings.
+
+    trace_path may be an iterable (e.g., array, map from pathlib.Path.glob()).
+
+    When multiple trace_paths are given, each one is handled iteratively, but
+    the processing of each stream is farmed out to a worker.
+    """
+    dirs = []
+    received = 0
+    discarded = 0
+
+    if isinstance(trace_path, str) or isinstance(trace_path, pathlib.Path):
+        dirs.append(str(trace_path))
+    else:
+        dirs = trace_path
+
+    for dir in dirs:
+        streams_path = None
+        for root, dirs, files in os.walk(str(dir)):
+            if "metadata" in files:
+                streams_path = pathlib.Path(root)
+                break
+
+        if streams_path is None:
+            raise RuntimeError("No metadata found in trace path: {}".format(trace_path))
+
+        metadata_path = streams_path / "metadata"
+        stream_dirs = []
+
+        for file in streams_path.iterdir():
+            if file.name == "metadata":
+                continue
+
+            temp_dir_path = test_env.create_temporary_directory()
+
+            # Symlink to the stream file
+            stream_link = temp_dir_path / file.name
+            stream_link.symlink_to(file)
+            # Symlink to the metadata file
+            metadata_link = temp_dir_path / "metadata"
+            metadata_link.symlink_to(metadata_path)
+
+            stream_dirs.append(temp_dir_path)
+
+        with multiprocessing.Pool() as pool:
+            results = pool.map(count_events_worker, stream_dirs)
+
+        for this_received, this_discarded in results:
+            received += this_received
+            discarded += this_discarded
+
+    return received, discarded
+
+
 @contextlib.contextmanager
 def kernel_module(module_name):
     """
