@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+ * SPDX-FileCopyrightText: 2011 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
  * SPDX-License-Identifier: MIT
  *
@@ -12,10 +12,12 @@
 #define _TRACEPOINT_TP_H
 
 #include <lttng/tracepoint.h>
+
 #include <stdint.h>
 
-TRACEPOINT_ENUM(
-	tp, tptest_enum,
+/* clang-format off */
+TRACEPOINT_ENUM(tp,
+	tptest_enum,
 	TP_ENUM_VALUES(
 		ctf_enum_auto("AUTO: EXPECT 0")
 		ctf_enum_value("VALUE: 23", 23)
@@ -27,11 +29,31 @@ TRACEPOINT_ENUM(
 	)
 )
 
+TRACEPOINT_ENUM(tp,
+	tptest_multi_range_enum,
+	TP_ENUM_VALUES(
+		ctf_enum_range("multiple ranges", 0, 99)
+		ctf_enum_value("entry a", 100)
+		ctf_enum_range("multiple ranges", 101, 199)
+		ctf_enum_value("entry b", 200)
+		ctf_enum_range("multiple ranges", 201, 299)
+		ctf_enum_value("entry c", 300)
+		ctf_enum_range("multiple ranges", 301, 1023)
+	)
+)
+
 TRACEPOINT_EVENT(tp, tptest,
-	TP_ARGS(int, anint, int, netint, long *, values,
-		char *, text, size_t, textlen,
-		char *, etext, uint32_t * , net_values,
-		double, doublearg, float, floatarg),
+	TP_ARGS(
+		int, anint,
+		int, netint,
+		long *, values,
+		const char *, text,
+		size_t, textlen,
+		const char *, etext,
+		uint32_t *, net_values,
+		double, doublearg,
+		float, floatarg
+	),
 	TP_FIELDS(
 		ctf_integer(int, intfield, anint)
 		ctf_integer_hex(int, intfield2, anint)
@@ -57,14 +79,25 @@ TRACEPOINT_EVENT(tp, tptest,
 		ctf_enum(tp, tptest_enum, int, enum202, 202)
 		ctf_enum(tp, tptest_enum, int, enum304, 304)
 		ctf_enum(tp, tptest_enum, int, enumnegative, -1)
+		/* Match 'multiple ranges'. */
+		ctf_enum(tp, tptest_multi_range_enum, int, enum50, 50)
+		/* Match 'entry a'. */
+		ctf_enum(tp, tptest_multi_range_enum, int, enum100, 100)
+		/* Match 'multiple ranges'. */
+		ctf_enum(tp, tptest_multi_range_enum, int, enum150, 150)
+		/* Match 'entry b'. */
+		ctf_enum(tp, tptest_multi_range_enum, int, enum200, 200)
+		/* Match 'multiple ranges'. */
+		ctf_enum(tp, tptest_multi_range_enum, int, enum250, 250)
+		/* Match 'entry c'. */
+		ctf_enum(tp, tptest_multi_range_enum, int, enum300, 300)
+		/* Match 'multiple ranges'. */
+		ctf_enum(tp, tptest_multi_range_enum, int, enum500, 500)
 	)
 )
+/* clang-format on */
 
-TRACEPOINT_EVENT(tp, end,
-	TP_ARGS(),
-	TP_FIELDS(
-	)
-)
+TRACEPOINT_EVENT(tp, end, TP_ARGS(), TP_FIELDS())
 
 #endif /* _TRACEPOINT_TP_H */
 
